unit UFormAcronym;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  StdCtrls, ActnList, UAcronym;

type

  { TFormAcronym }

  TFormAcronym = class(TForm)
    ACategoryRemove: TAction;
    ACategoryAdd: TAction;
    ActionList1: TActionList;
    Button1: TButton;
    Button2: TButton;
    ButtonOk: TButton;
    ButtonCancel: TButton;
    EditAcronym: TEdit;
    EditMeaning: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    ListBoxCategories: TListBox;
    ListBoxImportSources: TListBox;
    MemoDescription: TMemo;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    PopupMenuCategory: TPopupMenu;
    procedure ACategoryAddExecute(Sender: TObject);
    procedure ACategoryRemoveExecute(Sender: TObject);
    procedure EditMeaningKeyPress(Sender: TObject; var Key: char);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListBoxCategoriesSelectionChange(Sender: TObject; User: boolean);
  private
    procedure UpdateInterface;
  public
    procedure Load(Entry: TAcronymEntry);
    procedure Save(Entry: TAcronymEntry);
  end;

var
  FormAcronym: TFormAcronym;

implementation

{$R *.lfm}

uses
  UCore, UFormCategorySelect;

{ TFormAcronym }

procedure TFormAcronym.FormClose(Sender: TObject; var CloseAction: TCloseAction
  );
begin
  Core.PersistentForm1.Save(Self);
end;

procedure TFormAcronym.FormCreate(Sender: TObject);
begin
  Core.CoolTranslator1.TranslateComponentRecursive(Self);
end;

procedure TFormAcronym.ACategoryAddExecute(Sender: TObject);
var
  I: Integer;
begin
  FormCategorySelect := TFormCategorySelect.Create(Self);
  try
    FormCategorySelect.Load(ListBoxCategories.Items);
    if FormCategorySelect.ShowModal = mrOk then begin
      for I := 0 to FormCategorySelect.ListBox1.Count - 1 do
        if FormCategorySelect.ListBox1.Selected[I] then begin
          ListBoxCategories.Items.AddObject(FormCategorySelect.ListBox1.Items[I], FormCategorySelect.ListBox1.Items.Objects[I]);
        end;
    end;
  finally
    FreeAndNil(FormCategorySelect);
  end;
end;

procedure TFormAcronym.ACategoryRemoveExecute(Sender: TObject);
var
  I: Integer;
begin
  if MessageDlg(SRemoveCategory, SRemoveCategoryQuery,
  TMsgDlgType.mtConfirmation, [mbCancel, mbOk], 0) = mrOk then begin
    for I := ListBoxCategories.Items.Count - 1 downto 0 do
    if ListBoxCategories.Selected[I] then
      ListBoxCategories.Items.Delete(I);
    UpdateInterface;
  end;
end;

procedure TFormAcronym.EditMeaningKeyPress(Sender: TObject; var Key: char);
begin
  if Ord(Key) = 13 then ButtonOk.Click;
  if Ord(Key) = 27 then ButtonCancel.Click;
end;

procedure TFormAcronym.FormShow(Sender: TObject);
begin
  Core.PersistentForm1.Load(Self);
  UpdateInterface;
end;

procedure TFormAcronym.ListBoxCategoriesSelectionChange(Sender: TObject; User: boolean);
begin
  UpdateInterface;
end;

procedure TFormAcronym.UpdateInterface;
begin
  ACategoryRemove.Enabled := ListBoxCategories.ItemIndex <> -1;
end;

procedure TFormAcronym.Load(Entry: TAcronymEntry);
begin
  EditAcronym.Text := Entry.Name;
  EditMeaning.Text := Entry.Meaning;
  MemoDescription.Text := Entry.Description;
  ListBoxCategories.Items.Assign(Entry.Categories);
  ListBoxImportSources.Items.Assign(Entry.Sources);
end;

procedure TFormAcronym.Save(Entry: TAcronymEntry);
begin
  Entry.Name := EditAcronym.Text;
  Entry.Meaning := EditMeaning.Text;
  Entry.Description := MemoDescription.Text;
  Entry.Categories.Assign(ListBoxCategories.Items);
end;

end.

