unit UFormImportPattern;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  UAcronym;

type

  { TFormImportPattern }

  TFormImportPattern = class(TForm)
    ButtonCancel: TButton;
    ButtonOk: TButton;
    CheckBoxRepetition: TCheckBox;
    ComboBoxAction: TComboBox;
    ComboBoxVariable: TComboBox;
    EditBlockEnd: TEdit;
    EditBlockStart: TEdit;
    Label10: TLabel;
    Label11: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    procedure InitControls;
  public
    procedure Save(Pattern: TImportPattern);
    procedure Load(Pattern: TImportPattern);
  end;

var
  FormImportPattern: TFormImportPattern;

implementation

{$R *.lfm}

uses
  UCore;

{ TFormImportPattern }

procedure TFormImportPattern.FormShow(Sender: TObject);
begin
  InitControls;
end;

procedure TFormImportPattern.FormCreate(Sender: TObject);
begin
  Core.CoolTranslator1.TranslateComponentRecursive(Self);
  InitControls;
end;

procedure TFormImportPattern.InitControls;
var
  I: TImportPatternFlag;
  J: TImportVariable;
  Index: Integer;
begin
  Index := ComboBoxAction.ItemIndex;
  while ComboBoxAction.Items.Count > 0 do
    ComboBoxAction.Items.Delete(ComboBoxAction.Items.Count - 1);
  while ComboBoxAction.Items.Count <= Integer(High(ImportPatternFlagString)) do
    ComboBoxAction.Items.Add('');
  for I := Low(TImportPatternFlag) to High(TImportPatternFlag) do
    ComboBoxAction.Items[Integer(I)] := ImportPatternFlagString[I];
  ComboBoxAction.ItemIndex := Index;

  Index := ComboBoxVariable.ItemIndex;
  ComboBoxVariable.Items.Clear;
  while ComboBoxVariable.Items.Count > 0 do
    ComboBoxVariable.Items.Delete(ComboBoxVariable.Items.Count - 1);
  while ComboBoxVariable.Items.Count <= Integer(High(ImportVariableString)) do
    ComboBoxVariable.Items.Add('');
  for J := Low(TImportVariable) to High(TImportVariable) do
    ComboBoxVariable.Items[Integer(J)] := ImportVariableString[J];
  ComboBoxVariable.ItemIndex := Index;
end;

procedure TFormImportPattern.Save(Pattern: TImportPattern);
begin
  Pattern.StartString := EditBlockStart.Text;
  Pattern.EndString := EditBlockEnd.Text;
  Pattern.Variable := TImportVariable(ComboBoxVariable.ItemIndex);
  Pattern.Flag := TImportPatternFlag(ComboBoxAction.ItemIndex);
  Pattern.Repetition := CheckBoxRepetition.Checked;
end;

procedure TFormImportPattern.Load(Pattern: TImportPattern);
begin
  EditBlockStart.Text := Pattern.StartString;
  EditBlockEnd.Text := Pattern.EndString;
  ComboBoxVariable.ItemIndex := Integer(Pattern.Variable);
  ComboBoxAction.ItemIndex := Integer(Pattern.Flag);
  CheckBoxRepetition.Checked := Pattern.Repetition;
end;

end.

