unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  ComCtrls, StdCtrls, ExtCtrls, ActnList, Grids, UAcronym, UPersistentForm,
  URegistry, ULastOpenedList, UListViewSort, UJobProgressView,
  Registry, SpecializedList, LazUTF8, LazFileUtils;

type

  { TFormMain }

  TFormMain = class(TForm)
    AFilterEnabledCategories: TAction;
    AManageAcronym: TAction;
    AFilterSameLetterCase: TAction;
    AFilterSameLength: TAction;
    AHide: TAction;
    AExport: TAction;
    AProcessImports: TAction;
    AShowImportFormats: TAction;
    AShowAbout: TAction;
    AShowImportSources: TAction;
    AShowAcronyms: TAction;
    AShowCategories: TAction;
    ASettings: TAction;
    AFileSaveAs: TAction;
    AFileClose: TAction;
    AFileSave: TAction;
    AFileNew: TAction;
    AFileOpen: TAction;
    AImport: TAction;
    AShow: TAction;
    AExit: TAction;
    ActionList1: TActionList;
    CheckBoxEnabledCategories: TCheckBox;
    CheckBoxExactLength: TCheckBox;
    CheckBoxCaseSensitive: TCheckBox;
    CoolBar1: TCoolBar;
    LastOpenedList1: TLastOpenedList;
    ListViewAcronyms: TListView;
    ListViewFilter1: TListViewFilter;
    ListViewSort1: TListViewSort;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItemStatusBar: TMenuItem;
    MenuItemParam: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem20: TMenuItem;
    MenuItem21: TMenuItem;
    MenuItem22: TMenuItem;
    MenuItem23: TMenuItem;
    MenuItem24: TMenuItem;
    MenuItem25: TMenuItem;
    MenuItemToolbar: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItemOpenRecent: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    PanelParam: TPanel;
    PanelMain: TPanel;
    PopupMenuFilter: TPopupMenu;
    PopupMenuOpenRecent: TPopupMenu;
    SaveDialog1: TSaveDialog;
    StatusBar1: TStatusBar;
    ToolBar1: TToolBar;
    ToolBar2: TToolBar;
    ToolBar3: TToolBar;
    ToolBar4: TToolBar;
    ToolButton1: TToolButton;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    ToolButton16: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    procedure AExitExecute(Sender: TObject);
    procedure AExportExecute(Sender: TObject);
    procedure AFileCloseExecute(Sender: TObject);
    procedure AFileNewExecute(Sender: TObject);
    procedure AFileOpenExecute(Sender: TObject);
    procedure AFileSaveAsExecute(Sender: TObject);
    procedure AFileSaveExecute(Sender: TObject);
    procedure AFilterEnabledCategoriesExecute(Sender: TObject);
    procedure AFilterSameLetterCaseExecute(Sender: TObject);
    procedure AFilterSameLengthExecute(Sender: TObject);
    procedure AHideExecute(Sender: TObject);
    procedure AImportExecute(Sender: TObject);
    procedure AManageAcronymExecute(Sender: TObject);
    procedure AProcessImportsExecute(Sender: TObject);
    procedure ASettingsExecute(Sender: TObject);
    procedure AShowAboutExecute(Sender: TObject);
    procedure AShowAcronymsExecute(Sender: TObject);
    procedure AShowCategoriesExecute(Sender: TObject);
    procedure AShowExecute(Sender: TObject);
    procedure AShowImportFormatsExecute(Sender: TObject);
    procedure AShowImportSourcesExecute(Sender: TObject);
    procedure EditSearchChange(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LastOpenedList1Change(Sender: TObject);
    procedure ListViewAcronymsData(Sender: TObject; Item: TListItem);
    procedure ListViewAcronymsResize(Sender: TObject);
    procedure ListViewAcronymsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure ListViewFilter1Change(Sender: TObject);
    procedure ListViewSort1ColumnWidthChanged(Sender: TObject);
    function ListViewSort1CompareItem(Item1, Item2: TObject): Integer;
    procedure ListViewSort1Filter(ListViewSort: TListViewSort);
    procedure MenuItemParamClick(Sender: TObject);
    procedure MenuItemStatusBarClick(Sender: TObject);
    procedure MenuItemToolbarClick(Sender: TObject);
  private
    RegistryContext: TRegistryContext;
    ProjectClosed: Boolean;
    ImportTotalItemCount: Integer;
    procedure AcronymDbUpdate(Sender: TObject);
    function FilterCell(Text1, Text2: string): Boolean;
    procedure ProcessImportsJob(Job: TJob);
    procedure FilterList(List: TListObject);
    procedure OpenRecentClick(Sender: TObject);
  public
    procedure LoadConfig;
    procedure SaveConfig;
    procedure UpdateAcronymsList;
    procedure UpdateInterface;
    procedure ProjectOpen(FileName: string);
    function CompareStrings(Strings1, Strings2: TStrings): Boolean;
  end;

var
  FormMain: TFormMain;

resourcestring
  SAddedCount = 'Imported %d acronyms. Added %d new.';
  SProcessImportSources = 'Process import sources';
  SAcronymsCount = 'Acronyms count: %d';
  SMeaningsCount = 'Meanings count: %d';


implementation

{$R *.lfm}

uses
  UFormImport, UFormSettings, UFormCategories, UFormAcronyms, UFormExport,
  UFormImportSources, UFormAbout, UFormImportFormats, UCore;

resourcestring
  SModified = 'modified';
  SAppExit = 'Application exit';
  SAppExitQuery = 'Acronyms were modified. Do you want to save them to file before exit?';

const
  ProjectExt = '.adp';
  DefaultFileName = 'Acronyms' + ProjectExt;
  RegistryRunKey = '\Software\Microsoft\Windows\CurrentVersion\Run';

{ TFormMain }

procedure TFormMain.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to ToolBar1.ButtonCount - 1 do
    ToolBar1.Buttons[I].Hint := ToolBar1.Buttons[I].Caption;
  for I := 0 to ToolBar2.ButtonCount - 1 do
    ToolBar2.Buttons[I].Hint := ToolBar2.Buttons[I].Caption;
  for I := 0 to ToolBar3.ButtonCount - 1 do
    ToolBar3.Buttons[I].Hint := ToolBar3.Buttons[I].Caption;
  for I := 0 to ToolBar4.ButtonCount - 1 do
    ToolBar4.Buttons[I].Hint := ToolBar4.Buttons[I].Caption;
  PanelMain.ControlStyle := PanelMain.ControlStyle + [csOpaque];
  PanelParam.ControlStyle := PanelParam.ControlStyle + [csOpaque];
end;

procedure TFormMain.EditSearchChange(Sender: TObject);
begin
  UpdateAcronymsList;
end;

procedure TFormMain.FormHide(Sender: TObject);
begin
  if Core.InitializeFinished then Core.PersistentForm1.Save(Self);
  UpdateInterface;
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.PersistentForm1.Save(Self);
  Core.SaveConfig;
end;

procedure TFormMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  ProjectClosed := True;
  AFileClose.Execute;
  CanClose := ProjectClosed;
end;

procedure TFormMain.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.AExportExecute(Sender: TObject);
begin
  FormExport := TFormExport.Create(Self);
  try
    FormExport.ShowModal;
  finally
    FreeAndNil(FormExport);
  end;
end;

procedure TFormMain.AFileCloseExecute(Sender: TObject);
var
  ModalResult: TModalResult;
  DoClose: Boolean;
begin
  DoClose := False;
  if Assigned(Core.AcronymDb) then begin
    if Core.AcronymDb.Modified then begin
       ModalResult := MessageDlg(SAppExit, SAppExitQuery,
       mtConfirmation, [mbYes, mbNo, mbCancel], 0);
      if ModalResult = mrYes then begin
        AFileSave.Execute;
        DoClose := True;
      end
      else if ModalResult = mrNo then begin
        DoClose := True;
      end else ProjectClosed := False;
    end else DoClose := True;
  end;
  if DoClose then begin
    FreeAndNil(Core.AcronymDb);
    UpdateAcronymsList;
    UpdateInterface;
    ProjectClosed := True;
  end;
end;

procedure TFormMain.AFileNewExecute(Sender: TObject);
begin
  AFileClose.Execute;
  if not Assigned(Core.AcronymDb)  then begin
    Core.AcronymDb := TAcronymDb.Create;
    Core.AcronymDb.FileName := DefaultFileName;
    Core.AcronymDb.Acronyms.Clear;
    Core.AcronymDb.OnUpdate.Add(AcronymDbUpdate);
    UpdateAcronymsList;
    UpdateInterface;
  end;
end;

procedure TFormMain.AFileOpenExecute(Sender: TObject);
begin
  OpenDialog1.DefaultExt := ProjectExt;
  if Assigned(Core.AcronymDb) then
    OpenDialog1.FileName := Core.AcronymDb.FileName;
  if OpenDialog1.Execute then begin
    ProjectOpen(OpenDialog1.FileName);
  end;
end;

procedure TFormMain.AFileSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.DefaultExt := ProjectExt;
  SaveDialog1.FileName := Core.AcronymDb.FileName;
  if SaveDialog1.Execute then begin
    Core.AcronymDb.SaveToFile(SaveDialog1.FileName);
    LastOpenedList1.AddItem(SaveDialog1.FileName);
    UpdateInterface;
  end;
end;

procedure TFormMain.AFileSaveExecute(Sender: TObject);
begin
  if FileExists(Core.AcronymDb.FileName) then begin
    Core.AcronymDb.SaveToFile(Core.AcronymDb.FileName);
    LastOpenedList1.AddItem(Core.AcronymDb.FileName);
    UpdateInterface;
  end else AFileSaveAs.Execute;
end;

procedure TFormMain.AFilterEnabledCategoriesExecute(Sender: TObject);
begin
  UpdateAcronymsList;
end;

procedure TFormMain.AFilterSameLetterCaseExecute(Sender: TObject);
begin
  UpdateAcronymsList;
end;

procedure TFormMain.AFilterSameLengthExecute(Sender: TObject);
begin
  UpdateAcronymsList;
end;

procedure TFormMain.AHideExecute(Sender: TObject);
begin
  Hide;
end;

procedure TFormMain.AImportExecute(Sender: TObject);
begin
  FormImport := TFormImport.Create(Self);
  try
    FormImport.ShowModal;
    UpdateAcronymsList;
    UpdateInterface;
  finally
    FreeAndNil(FormImport);
  end;
end;

procedure TFormMain.AManageAcronymExecute(Sender: TObject);
begin
  FormAcronyms := TFormAcronyms.Create(Self);
  try
    if Assigned(ListViewAcronyms.Selected) then
      FormAcronyms.FocusAcronym := ListViewAcronyms.Selected.Data;
    FormAcronyms.ShowModal;
    UpdateAcronymsList;
    UpdateInterface;
  finally
    FreeAndNil(FormAcronyms);
  end;
end;

procedure TFormMain.AProcessImportsExecute(Sender: TObject);
begin
  ImportTotalItemCount := 0;
  Core.AcronymDb.AddedCount := 0;
  Core.JobProgressView1.AddJob(SProcessImportSources, ProcessImportsJob);
  Core.JobProgressView1.Start;
  ShowMessage(Format(SAddedCount, [ImportTotalItemCount, Core.AcronymDb.AddedCount]));
  UpdateAcronymsList;
  UpdateInterface;
end;

procedure TFormMain.ProcessImportsJob(Job: TJob);
var
  I: Integer;
begin
  for I := 0 to Core.AcronymDb.ImportSources.Count - 1 do
  with TImportSource(Core.AcronymDb.ImportSources[I]) do
  if Enabled then begin
    Process;
    ImportTotalItemCount := ImportTotalItemCount + ItemCount;
    Job.Progress.Max := Core.AcronymDb.ImportSources.Count;
    Job.Progress.Value := I;
    if Job.Terminate then Break;
  end;
end;

procedure TFormMain.ASettingsExecute(Sender: TObject);
begin
  FormSettings := TFormSettings.Create(Self);
  try
    FormSettings.Load;
    if FormSettings.ShowModal = mrOk then begin
    FormSettings.Save;
      Core.SaveConfig;
    end;
  finally
    FreeAndNil(FormSettings);
  end;
end;

procedure TFormMain.AShowAboutExecute(Sender: TObject);
begin
  FormAbout := TFormAbout.Create(Self);
  try
    FormAbout.ApplicationInfo := Core.ApplicationInfo1;
    FormAbout.ShowModal;
  finally
    FreeAndNil(FormAbout);
  end;
end;

procedure TFormMain.AShowAcronymsExecute(Sender: TObject);
begin
  FormAcronyms := TFormAcronyms.Create(Self);
  try
    FormAcronyms.ShowModal;
    UpdateAcronymsList;
    UpdateInterface;
  finally
    FreeAndNil(FormAcronyms);
  end;
end;

procedure TFormMain.AShowCategoriesExecute(Sender: TObject);
begin
  FormCategories := TFormCategories.Create(Self);
  try
    FormCategories.Categories := Core.AcronymDb.Categories;
    FormCategories.ShowModal;
    UpdateAcronymsList;
    UpdateInterface;
  finally
    FreeAndNil(FormCategories);
  end;
end;

procedure TFormMain.AShowExecute(Sender: TObject);
begin
  Show;
  BringToFront;
end;

procedure TFormMain.AShowImportFormatsExecute(Sender: TObject);
begin
  FormImportFormats := TFormImportFormats.Create(Self);
  try
    FormImportFormats.ImportFormats := Core.AcronymDb.ImportFormats;
    FormImportFormats.ShowModal;
    UpdateInterface;
  finally
    FreeAndNil(FormImportFormats);
  end;
end;

procedure TFormMain.AShowImportSourcesExecute(Sender: TObject);
begin
  FormImportSources := TFormImportSources.Create(Self);
  try
    FormImportSources.ImportSources := Core.AcronymDb.ImportSources;
    FormImportSources.ShowModal;
    UpdateAcronymsList;
    UpdateInterface;
  finally
    FreeAndNil(FormImportSources);
  end;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  Core.Initialize;

  if Visible then begin
    Core.PersistentForm1.Load(Self);
    UpdateInterface;
    ListViewFilter1.StringGrid.Col := 1;
    ListViewFilter1.StringGrid.Col := 0;
    ListViewFilter1.StringGrid.Row := 0;
    ListViewFilter1.StringGrid.SetFocus;
  end;
end;

procedure TFormMain.LastOpenedList1Change(Sender: TObject);
begin
  LastOpenedList1.LoadToMenuItem(MenuItemOpenRecent, OpenRecentClick);
  LastOpenedList1.LoadToMenuItem(PopupMenuOpenRecent.Items, OpenRecentClick);
end;

procedure TFormMain.ListViewAcronymsData(Sender: TObject; Item: TListItem);
begin
  if Item.Index < ListViewSort1.List.Count then
  with TAcronymMeaning(ListViewSort1.List[Item.Index]) do begin
    Item.Caption := Acronym.Name;
    Item.Data := TAcronymMeaning(ListViewSort1.List[Item.Index]);
    Item.SubItems.Add(Name);
    Item.SubItems.Add(Categories.GetString);
  end;
end;

procedure TFormMain.ListViewAcronymsResize(Sender: TObject);
begin
  ListViewFilter1.UpdateFromListView(ListViewAcronyms);
end;

procedure TFormMain.ListViewAcronymsSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  UpdateInterface;
end;

procedure TFormMain.ListViewFilter1Change(Sender: TObject);
begin
  UpdateAcronymsList;
end;

procedure TFormMain.ListViewSort1ColumnWidthChanged(Sender: TObject);
begin
  ListViewFilter1.UpdateFromListView(ListViewAcronyms);
end;

function TFormMain.ListViewSort1CompareItem(Item1, Item2: TObject): Integer;
begin
  Result := 0;
  if Assigned(Item1) and Assigned(Item2) and (ListViewSort1.Order <> soNone) then begin
    with ListViewSort1 do
    case Column of
      0: Result := CompareString(TAcronymMeaning(Item1).Acronym.Name, TAcronymMeaning(Item2).Acronym.Name);
      1: Result := CompareString(TAcronymMeaning(Item1).Name, TAcronymMeaning(Item2).Name);
      2: Result := CompareString(TAcronymMeaning(Item1).Categories.GetString, TAcronymMeaning(Item2).Categories.GetString);
    end;
    if ListViewSort1.Order = soDown then Result := -Result;
  end else Result := 0;
end;

procedure TFormMain.ListViewSort1Filter(ListViewSort: TListViewSort);
begin
  Core.AcronymDb.AssignToList(ListViewSort1.List, AFilterEnabledCategories.Checked);
  FilterList(ListViewSort1.List);
end;

procedure TFormMain.MenuItemParamClick(Sender: TObject);
begin
  MenuItemParam.Checked := not MenuItemParam.Checked;
  UpdateInterface;
end;

procedure TFormMain.MenuItemStatusBarClick(Sender: TObject);
begin
  MenuItemStatusBar.Checked := not MenuItemStatusBar.Checked;
  UpdateInterface;
end;

procedure TFormMain.MenuItemToolbarClick(Sender: TObject);
begin
  MenuItemToolbar.Checked := not MenuItemToolbar.Checked;
  UpdateInterface;
end;

procedure TFormMain.AcronymDbUpdate(Sender: TObject);
begin
  UpdateAcronymsList;
end;

function TFormMain.FilterCell(Text1, Text2: string): Boolean;
begin
  if not AFilterSameLetterCase.Checked then begin
    Text1 := UTF8LowerCase(Text1);
    Text2 := UTF8LowerCase(Text2);
  end;
  if AFilterSameLength.Checked then
    Result := Text1 = Text2
    else Result := Pos(Text1, Text2) > 0;
end;

procedure TFormMain.ProjectOpen(FileName: string);
begin
  AFileClose.Execute;
  if not Assigned(Core.AcronymDb) then begin
    try
      AFileNew.Execute;
      Core.AcronymDb.LoadFromFile(FileName);
      LastOpenedList1.AddItem(FileName);
    finally
      UpdateAcronymsList;
      UpdateInterface;
    end;
  end;
end;

function TFormMain.CompareStrings(Strings1, Strings2: TStrings): Boolean;
var
  I: Integer;
begin
  Result := Strings1.Count = Strings2.Count;
  if not Result then Exit;
  for I := 0 to Strings1.Count - 1 do
    if (Strings1[I] <> Strings2[I]) or (Strings1.Objects[I] <> Strings2.Objects[I]) then begin
      Result := False;
      Exit;
    end;
end;

procedure TFormMain.FilterList(List: TListObject);
var
  I: Integer;
  FoundCount: Integer;
  EnteredCount: Integer;
begin
  EnteredCount := ListViewFilter1.TextEnteredCount;
  for I := List.Count - 1 downto 0 do begin
    if List.Items[I] is TAcronymMeaning then begin
      with TAcronymMeaning(List.Items[I]) do begin
         with ListViewFilter1 do
         if Visible and (EnteredCount > 0) then begin
           FoundCount := 0;
           if TextEnteredColumn(0) and (FilterCell(StringGrid.Cells[0, 0],
             TAcronymMeaning(List.Items[I]).Acronym.Name)) then Inc(FoundCount);
           if TextEnteredColumn(1) and (FilterCell(StringGrid.Cells[1, 0],
             TAcronymMeaning(List.Items[I]).Name)) then Inc(FoundCount);
           if TextEnteredColumn(2) and (FilterCell(StringGrid.Cells[2, 0],
             TAcronymMeaning(List.Items[I]).Categories.GetString)) then Inc(FoundCount);
           if FoundCount <> EnteredCount then List.Delete(I);
         end else List.Delete(I);
      end;
    end else
    if TAcronymMeaning(List.Items[I]) is TAcronymMeaning then begin
      List.Delete(I);
    end;
  end;
end;

procedure TFormMain.OpenRecentClick(Sender: TObject);
begin
  AFileClose.Execute;
  if not Assigned(Core.AcronymDb) then begin
    AFileNew.Execute;
    Core.AcronymDb.LoadFromFile(TMenuItem(Sender).Caption);
    LastOpenedList1.AddItem(TMenuItem(Sender).Caption);
    UpdateAcronymsList;
    UpdateInterface;
  end;
end;

procedure TFormMain.UpdateAcronymsList;
begin
  if Assigned(Core.AcronymDb) then begin
    ListViewSort1.Refresh;
  end else begin
    ListViewSort1.List.Clear;
    ListViewAcronyms.Items.Count := 0;
    ListViewAcronyms.Refresh;
  end;
end;

procedure TFormMain.UpdateInterface;
var
  Title: string;
begin
  ListViewAcronyms.Enabled := Assigned(Core.AcronymDb);
  AFileClose.Enabled := Assigned(Core.AcronymDb);
  AFileSave.Enabled := Assigned(Core.AcronymDb) and Core.AcronymDb.Modified;
  AFileSaveAs.Enabled := Assigned(Core.AcronymDb);
  AImport.Enabled := Assigned(Core.AcronymDb);
  AExport.Enabled := Assigned(Core.AcronymDb);
  AProcessImports.Enabled := Assigned(Core.AcronymDb);
  AShowCategories.Enabled := Assigned(Core.AcronymDb);
  AShowAcronyms.Enabled := Assigned(Core.AcronymDb);
  AShowCategories.Enabled := Assigned(Core.AcronymDb);
  AShowImportSources.Enabled := Assigned(Core.AcronymDb);
  AShowImportFormats.Enabled := Assigned(Core.AcronymDb);
  CoolBar1.Visible := MenuItemToolbar.Checked;
  PanelParam.Visible := MenuItemParam.Checked;
  AHide.Enabled := FormMain.Visible;
  AManageAcronym.Enabled := Assigned(ListViewAcronyms.Selected);
  StatusBar1.Visible := MenuItemStatusBar.Checked;

  Title := '';
  if Assigned(Core.AcronymDb) and (ExtractFileNameWithoutExt(ExtractFileName(Core.AcronymDb.FileName)) <> '') then begin
    Title := ExtractFileNameWithoutExt(ExtractFileName(Core.AcronymDb.FileName));
    if Core.AcronymDb.Modified then Title := Title + ' (' + SModified + ')';
  end;
  if Title <> '' then Title := Title + ' - ';
  Title := Title + Core.ApplicationInfo1.AppName;
  Application.Title := Title;
  Caption := Title;
  if Assigned(Core.AcronymDb) then begin
    StatusBar1.Panels[0].Text := Format(SAcronymsCount, [
      Core.AcronymDb.Acronyms.Count]);
    StatusBar1.Panels[1].Text := Format(SMeaningsCount, [
      Core.AcronymDb.GetMeaningsCount]);
  end else begin
    StatusBar1.Panels[0].Text := '';
    StatusBar1.Panels[1].Text := '';
  end;
end;

procedure TFormMain.LoadConfig;
begin
  RegistryContext := RegContext(HKEY_CURRENT_USER, DefaultRegKey);
  Core.PersistentForm1.RegistryContext := RegistryContext;
  RegistryContext := RegContext(HKEY_CURRENT_USER, DefaultRegKey + '\RecentFiles');
  LastOpenedList1.LoadFromRegistry(RegistryContext);

  {$IFDEF Linux}
  // If installed in Linux system then use installation directory for po files
  if Application.ExeName = '/usr/bin/' + ExtractFileNameOnly(Application.ExeName) then
    Core.CoolTranslator1.POFilesFolder := '/usr/share/' + ExtractFileNameOnly(Application.ExeName) + '/languages';
  {$ENDIF}

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(DefaultRegKey, True);
    AFilterSameLength.Checked := ReadBoolWithDefault('SameLength', False);
    AFilterSameLetterCase.Checked := ReadBoolWithDefault('SameLetterCase', False);
    AFilterEnabledCategories.Checked := ReadBoolWithDefault('EnabledCategories', False);
    MenuItemToolbar.Checked := ReadBoolWithDefault('ToolBarVisible', True);
    MenuItemStatusBar.Checked := ReadBoolWithDefault('StatuslBarVisible', True);
    MenuItemParam.Checked := ReadBoolWithDefault('FilterParamVisible', True);
  finally
    Free;
  end;

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegistryRunKey, True);
    Core.StartOnLogon := ValueExists('Acronym Decoder');
  finally
    Free;
  end;
end;

procedure TFormMain.SaveConfig;
begin
  RegistryContext := RegContext(HKEY_CURRENT_USER, DefaultRegKey + '\RecentFiles');
  LastOpenedList1.SaveToRegistry(RegistryContext);

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(DefaultRegKey, True);
    WriteBool('SameLength', AFilterSameLength.Checked);
    WriteBool('SameLetterCase', AFilterSameLetterCase.Checked);
    WriteBool('EnabledCategories', AFilterEnabledCategories.Checked);
    WriteBool('ToolBarVisible', MenuItemToolbar.Checked);
    WriteBool('StatusBarVisible', MenuItemStatusBar.Checked);
    WriteBool('FilterParamVisible', MenuItemParam.Checked);
  finally
    Free;
  end;

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegistryRunKey, True);
    if Core.StartOnLogon then begin
      if Core.StartMinimizedToTray then WriteString('Acronym Decoder', Application.ExeName + ' ' + Application.OptionChar + 't')
        else WriteString('Acronym Decoder', Application.ExeName)
    end else DeleteValue('Acronym Decoder');
  finally
    Free;
  end;
end;

end.

